# -*- coding: utf-8 -*-
""" Module of sampling of continuous Hermite function computation

Ported from ltfat_2.1.0/comp/comp_hermite.m

.. moduleauthor:: Denis Arrivault
"""

from __future__ import print_function, division

import numpy as np
import math


def comp_hermite(n, x):
    """ Compute sampling of continuous Hermite function.

    - Usage:

        | ``y = comp_hermite(n, x)``

    `comp_hermite(n, x)` evaluates the n-th Hermite function at the vector
    **x**. The function is normalized to have the :math:`L^2` norm
    on :math:`]-\\infty,\\infty[` equal to one.

    A minimal effort is made to avoid underflow in recursion.
    If used to evaluate the Hermite quadratures, it works for n <= 2400
    """
    rt = 1 / math.sqrt(math.sqrt(math.pi))

    if n == 0:
        y = rt * np.exp(-0.5 * x**2)
    if n == 1:
        y = rt * math.sqrt(2) * x * np.exp(-0.5 * x**2)

    # if n > 2, conducting the recursion.
    if n >= 2:
        ef = np.exp(-0.5 * (x**2) / (n+1))
        tmp1 = rt * ef
        tmp2 = rt * math.sqrt(2) * x * (ef**2)
        for k in range(2, n+1):
            y = math.sqrt(2)*x*tmp2 - math.sqrt(k-1)*tmp1*ef
            y = ef * y / math.sqrt(k)
            tmp1 = tmp2
            tmp2 = y
    return y
