# -*- coding: utf-8 -*-
""" Module of sampling of all Hermite function computation

Ported from ltfat_2.1.0/comp/comp_hermite_all.m

.. moduleauthor:: Denis Arrivault
"""

from __future__ import print_function, division

import numpy as np


def comp_hermite_all(n, x):
    """ Compute all Hermite functions up to an order

    - Usage:

        | ``y = comp_hermite_all(n, x)``

    This function evaluates the Hermite functions
    of degree 0 through n-1 at the vector x.
    The functions are normalized to have the :math:`L^2` norm
    on :math:`]-\\infty,\\infty[` equal to one. No effort is made to
    avoid unerflow during recursion.

    - Input parameters:

    :param int n: the number of Hermite functions
    :param numpy.ndarray x: the vector of arguments

    - Output parameters:

    :return numpy.ndarray y: the values of the first n Hermite functions at
                             the nodes x
    """
    if not isinstance(x, np.ndarray) or len(x.shape) > 1:
        raise TypeError("x should be a numpy array of dimension 1")
    rt = 1 / np.sqrt(np.sqrt(np.pi))

    # conducting the recursion.
    y = np.zeros((len(x), n))

    if n == 0:
        y = rt * np.exp(-0.5 * x**2)
    else:
        y[:, 0] = rt * np.exp(-0.5 * x**2)
        if n > 1:
            y[:, 1] = rt * np.sqrt(2) * x * np.exp(-0.5 * x**2)
            for k in range(2, n):
                y[:, k] = np.sqrt(2)*x*y[:, k-1] - np.sqrt(k-1)*y[:, k-2]
                y[:, k] = y[:, k] / np.sqrt(k)
    return y
