# -*- coding: utf-8 -*-

"""This module contains interface functions for the LTFAT computed
versions of isepdgtreal calculations.

.. moduleauthor:: Denis Arrivault
"""

from __future__ import print_function, division

import cython
from cython.view cimport array as cvarray
from libc.stdlib cimport malloc, free
import numpy as np

from ltfat cimport ltfatInt, dgt_phasetype, idgtreal_fb_d, idgtreal_long_d
from ltfatpy.comp.ltfat cimport TIMEINV, FREQINV

# don’t check for out-of-bounds indexing.
@cython.boundscheck(False)
# assume no negative indexing.
@cython.wraparound(False)
cdef double[:] comp_idgtreal_long_d(const double complex[:] coef,
                                    const double[:] g,
                                    const int L, const int W,
                                    const int a, const int M,
                                    const dgt_phasetype ptype):
    """ Internal function, do not use it """
    cdef double * res
    cdef ltfatInt size_f = L * W
    res = < double * > malloc(sizeof(double) * size_f)

    idgtreal_long_d(& coef[0], & g[0], L, W, a, M, ptype, res)
    return < double[:size_f] > res

# don’t check for out-of-bounds indexing.
@cython.boundscheck(False)
# assume no negative indexing.
@cython.wraparound(False)
cdef double[:] comp_idgtreal_fb_d(const double complex[:] coef,
                                  const double[:] g, const int L, const int gl,
                                  const int W, const int a, const int M,
                                  const dgt_phasetype ptype):
    """ Internal function, do not use it """
    cdef double * res
    cdef ltfatInt size_sig = L * W
    res = < double* > malloc(sizeof(double) * size_sig)
    idgtreal_fb_d(& coef[0], & g[0], L, gl, W, a, M, ptype, res)
    return < double[:size_sig] > res

# don’t check for out-of-bounds indexing.
@cython.boundscheck(False)
# assume no negative indexing.
@cython.wraparound(False)
cpdef comp_isepdgtreal(coef, g, a, M, pt):
    """Function that computes separable inverse real dgt

    This is a computational subroutine, do not call it directly, use
    :func:`~ltfatpy.gabor.idgtreal.idgtreal` instead.
    
    .. seealso:: :func:`~ltfatpy.gabor.idgt.idgt`
    """

    if (coef.dtype.type != np.complex128):
        raise TypeError("coef data should be numpy.complex128")
    if (g.dtype.type != np.float64):
        raise TypeError("g data should be numpy.float64")
    if pt != 0 and pt != 1:
        raise TypeError("pt should be 0 (FREQINV) or 1 (TIMEINV)")
    if coef.ndim <= 1 or coef.ndim > 3:
        raise TypeError("Dimensions of dgt coefficients array must be 2 or 3.")
    cdef ltfatInt N = coef.shape[1]
    cdef ltfatInt L = N * a
    cdef ltfatInt M2 = coef.shape[0]
    if M2 != (M//2 + 1):
        raise TypeError("Mismatch between the specified number of channels " +
                        "and the size of the input coefficients.")
    cdef ltfatInt W = coef.size // (M2 * N)
    if W > 1 and coef.ndim == 2:
        raise TypeError("Dimensions of dgt coefficients array do not fit.")
    if W == 1 and coef.ndim == 3:
        coef = np.squeeze(coef, axis=2)

    coef_combined = coef.reshape(coef.size, order='F')

    cdef ltfatInt gl = g.shape[0]

    if g.ndim > 1:
        if g.ndim > 2:
            g = np.squeeze(g, axis=range(2, g.ndim-1))
        if g.ndim == 2:
            gl = gl * g.shape[1]
        g_combined = g.reshape(gl, order='F')
    else:
        g_combined = g

    if gl < L:
        res = np.asarray(comp_idgtreal_fb_d(coef_combined, g_combined, L, gl,
                                            W, a, M, pt))
        if W > 1:
            return np.reshape(res, (L, W), order='F')
        return res
    else:
        res = np.asarray(comp_idgtreal_long_d(coef_combined, g_combined, L, W,
                                              a, M, pt))
        if W > 1:
            return np.reshape(res, (L, W), order='F')
        return res
