# -*- coding: utf-8 -*-
"""This module contains interface functions for the LTFAT computed
versions of periodized Gaussian calculation.

.. moduleauthor:: Denis Arrivault
"""

from __future__ import print_function, division

import cython
from cython.view cimport array as cvarray
from libc.stdlib cimport malloc, free
import numpy as np

from ltfat cimport ltfatInt, pgauss_d, pgauss_cmplx_d

# don’t check for out-of-bounds indexing.
@cython.boundscheck(False)
# assume no negative indexing.
@cython.wraparound(False)
cdef double[:] comp_pgauss_real(ltfatInt L, double w, double c_t):
    """ Internal function, do not use it """
    cdef double * c_array = < double * > malloc(sizeof(double) * L)
    pgauss_d(L, w, c_t, c_array)
    return < double[:L] > c_array

# don’t check for out-of-bounds indexing.
@cython.boundscheck(False)
# assume no negative indexing.
@cython.wraparound(False)
cdef double complex[:] comp_pgauss_cplx(ltfatInt L, double w, double c_t,
                                        double c_f):
    """ Internal function, do not use it """
    cdef double complex * c_array = < double complex * > malloc(sizeof(double
                                                                complex) * L)
    pgauss_cmplx_d(L, w, c_t, c_f, c_array)
    return < double complex[:L] > c_array


cpdef comp_pgauss(L, w=1.0, c_t=0, c_f=0):
    """Return the computed sampled, periodized Gaussian.

    This is a computational subroutine, do not call it directly, use
    :func:`~ltfatpy.fourier.pgauss.pgauss` instead.
    """
    if c_f == 0:
        return np.asarray(comp_pgauss_real(L, w, c_t))
    return np.asarray(comp_pgauss_cplx(L, w, c_t, c_f))
