# -*- coding: utf-8 -*-
""" Module of gabpars_from_windowsignal calculation

Ported from ltfat_2.1.0/comp/gabpars_from_windowsignal.m

.. moduleauthor:: Denis Arrivault
"""

from __future__ import print_function, division

from ltfatpy.comp.comp_sigreshape_pre import comp_sigreshape_pre
from ltfatpy.gabor.dgtlength import dgtlength
from ltfatpy.gabor.gabwin import gabwin
from ltfatpy.tools.postpad import postpad


def gabpars_from_windowsignal(f, g, a, M, L=None):
    """Compute g and L from window and signal

    - Usage:

        | ``(g, info, L) = gabpars_from_windowsignal(f, g, a, M, L)``

    Use this function if you know an input signal and a window
    for the DGT. The function will calculate a transform length L and
    evaluate the window g into numerical form. The signal will be padded and
    returned as a column vector.

    If the transform length is unknown (as it usually is unless explicitly
    specified by the user), set L to be None in the input to this function.
    """
    # Verify f and determine its length
    # Change f to correct shape.
    (f, Ls, W, unused_wasrow, unused_remembershape) = comp_sigreshape_pre(f, 0)

    if L is None:
        # Verify a, M and get L from the signal length f
        L = dgtlength(Ls, a, M)
    else:
        # Verify a, M and get L
        Luser = dgtlength(L, a, M)
        if Luser != L:
            raise ValueError('Incorrect transform length L={0:d} specified.' +
                             'Next valid length is L={1:d}.'.format(L, Luser))

    # Determine the window
    (g, info) = gabwin(g, a, M, L)

    if L < info['gl']:
        raise ValueError('Window is too long.')

    # final cleanup
    f = postpad(f, L)

    # If the signal is single precision, make the window single precision as
    # well to avoid mismatches.
    # if isa(f,'single')
    # g=single(g);
    # end;
    return (f, g, L, Ls, W, info)
