# -*- coding: utf-8 -*-
"""This module contains dctIV function

Ported from ltfat_2.1.0/fourier/dctiv.m

.. moduleauthor:: Denis Arrivault
"""

from __future__ import print_function, division

from ltfatpy.comp.comp_dct import comp_dct
from ltfatpy.comp.assert_sigreshape_pre import assert_sigreshape_pre
from ltfatpy.comp.assert_sigreshape_post import assert_sigreshape_post
from ltfatpy.tools.postpad import postpad


def dctiv(f, L=None, dim=None):
    """Discrete Cosine Transform type IV

    - Usage:

        | ``c = dctiv(f)``
        | ``c = dctiv(f,L,dim)``

    - Input parameters:

    :param numpy.ndarray f: Input data. **f** dtype has to be float64 or
        complex128.
    :param int L: Length of the output vector. Default is the length of
        **f**.
    :param int dim: dimension along which the transformation is applied.
        Default is the first non-singleton dimension.

    - Output parameter:

    :return: ``c``
    :rtype: numpy.ndarray

    ``dctiv(f)`` computes the discrete cosine transform of type IV of the
    input signal **f**. If **f** is a matrix then the transformation is applied
    to each column. For N-D arrays, the transformation is applied to the first
    non-singleton dimension.

    ``dctiv(f,L)`` zero-pads or truncates **f** to length **L** before doing
    the transformation.

    ``dctiv(f,dim=dim)`` or ``dctiv(f,L,dim)`` applies the transformation
    along dimension **dim**.

    The transform is real (output is real if input is real) and
    it is orthonormal. It is its own inverse.

    Let f be a signal of length **L** and let :math:`c=dctiv(f)` then:

    .. math::

        c\\left(n+1\\right)=\\sqrt{\\frac{2}{L}}\\sum_{m=0}^{L-1}f\\left(
        m+1\\right)\\cos\\left(\\frac{\\pi}{L}\\left(n+\\frac{1}{2}\\right)
        \\left(m+\\frac{1}{2}\\right)\\right)

    - Examples:

    The following figures show the first 4 basis functions of the dctiv of
    length 20:

    >>> import numpy as np
    >>> # The dctiv is its own adjoint.
    >>> F = dctiv(np.eye(20, dtype=np.float64))
    >>> import matplotlib.pyplot as plt
    >>> plt.close('all')
    >>> fig = plt.figure()
    >>> for ii in range(1,5):
    ...    ax = fig.add_subplot(4,1,ii)
    ...    ax.stem(F[:,ii-1])
    ...
    <Container object of 3 artists>
    <Container object of 3 artists>
    <Container object of 3 artists>
    <Container object of 3 artists>
    >>> plt.show()

    .. image:: images/dctiv.png
       :width: 700px
       :alt: dctiv image
       :align: center
    .. seealso::  :func:`~ltfatpy.fourier.dctii`,
        :func:`~ltfatpy.fourier.dctiii`, :func:`~ltfatpy.fourier.dstiv`

    - References:
        :cite:`rayi90,wi94`
    """
    (f, L, _, _, dim, permutedsize, order) = assert_sigreshape_pre(f, L, dim)
    if L is not None:
        f = postpad(f, L)
    if L == 1:
        c = f
    else:
        c = comp_dct(f, 4)
    return assert_sigreshape_post(c, dim, permutedsize, order)

if __name__ == '__main__':
    import doctest
    doctest.testmod()
