# -*- coding: utf-8 -*-
"""This module contains DSTI function

Ported from ltfat_2.1.0/fourier/dsti.m

.. moduleauthor:: Denis Arrivault
"""

from __future__ import print_function, division

from ltfatpy.comp.comp_dst import comp_dst
from ltfatpy.comp.assert_sigreshape_pre import assert_sigreshape_pre
from ltfatpy.comp.assert_sigreshape_post import assert_sigreshape_post
from ltfatpy.tools.postpad import postpad


def dsti(f, L=None, dim=None):
    """Discrete Sine Transform type I

    - Usage:

        | ``c = dsti(f)``
        | ``c = dsti(f,L,dim)``

    - Input parameters:

    :param numpy.ndarray f: Input data. **f** dtype has to be float64 or
        complex128.
    :param int L: Length of the output vector. Default is the length of
        **f**.
    :param int dim: dimension along which the transformation is applied.
        Default is the first non-singleton dimension.

    - Output parameter:

    :return: ``c``
    :rtype: numpy.ndarray

    ``dsti(f)`` computes the discrete sine transform of type I of the
    input signal **f**. If **f** is multi-dimensional, the transformation is
    applied along the first non-singleton dimension.

    ``dsti(f,L)`` zero-pads or truncates **f** to length **L** before doing the
    transformation.

    ``dsti(f,[],dim)`` or ``dsti(f,L,dim)`` applies the transformation along
    dimension **dim**.

    The transform is real (output is real if input is real) and orthonormal.

    This transform is its own inverse.

    Let f be a signal of length **L** and let ``c=dsti(f)``. Then

    .. math::

        c\\left(n+1\\right)=\\sqrt{\\frac{2}{L+1}}\\sum_{m=0}^{L-1}f\\left(
        m+1\\right)\\sin\\left(\\frac{\\pi \\left(n+1\\right)\\left(
        m+1\\right)}{L+1}\\right)

    The implementation of this functions uses a simple algorithm that requires
    an FFT of length $2N+2$, which might potentially be the product of a large
    prime number. This may cause the function to sometimes execute slowly.
    If guaranteed high speed is a concern, please consider using one of the
    other DST transforms.

    - Examples:

    The following figures show the first 4 basis functions of the DSTI of
    length 20:

    >>> import numpy as np
    >>> # The dsti is its own adjoint.
    >>> F = dsti(np.eye(20, dtype=np.float64))
    >>> import matplotlib.pyplot as plt
    >>> plt.close('all')
    >>> fig = plt.figure()
    >>> for ii in range(1,5):
    ...    ax = fig.add_subplot(4,1,ii)
    ...    ax.stem(F[:,ii-1])
    ...
    <Container object of 3 artists>
    <Container object of 3 artists>
    <Container object of 3 artists>
    <Container object of 3 artists>
    >>> plt.show()

    .. image:: images/dsti.png
       :width: 700px
       :alt: dsti image
       :align: center
    .. seealso::  :func:`~ltfatpy.fourier.dstii`,
        :func:`~ltfatpy.fourier.dstiii`, :func:`~ltfatpy.fourier.dstiv`,
        :func:`~ltfatpy.fourier.dcti`

    - References:
        :cite:`rayi90,wi94`
    """
    (f, L, _, _, dim, permutedsize, order) = assert_sigreshape_pre(f, L, dim)
    if L is not None:
        f = postpad(f, L)
    if L == 1:
        c = f
    else:
        c = comp_dst(f, 1)
    return assert_sigreshape_post(c, dim, permutedsize, order)

if __name__ == '__main__':
    import doctest
    doctest.testmod()
