# -*- coding: utf-8 -*-
"""This module contains DSTII function

Ported from ltfat_2.1.0/fourier/dstii.m

.. moduleauthor:: Denis Arrivault
"""

from __future__ import print_function, division

from ltfatpy.comp.comp_dst import comp_dst
from ltfatpy.comp.assert_sigreshape_pre import assert_sigreshape_pre
from ltfatpy.comp.assert_sigreshape_post import assert_sigreshape_post
from ltfatpy.tools.postpad import postpad


def dstii(f, L=None, dim=None):
    """Discrete Sine Transform type II

    - Usage:

        | ``c = dstii(f)``
        | ``c = dstii(f,L,dim)``

    - Input parameters:

    :param numpy.ndarray f: Input data. **f** dtype has to be float64 or
        complex128.
    :param int L: Length of the output vector. Default is the length of
        **f**.
    :param int dim: dimension along which the transformation is applied.
        Default is the first non-singleton dimension.

    - Output parameter:

    :return: ``c``
    :rtype: numpy.ndarray

    ``dstii(f)`` computes the discrete sine transform of type I of the
    input signal **f**. If **f** is multi-dimensional, the transformation is
    applied along the first non-singleton dimension.

    ``dstii(f,L)`` zero-pads or truncates **f** to length **L** before doing
    the transformation.

    ``dstii(f,[],dim)`` or ``dstii(f,L,dim)`` applies the transformation along
    dimension **dim**.

    The transform is real (output is real if input is real) and orthonormal.

    The inverse transform of \|dstii\| is \|dstiii\|.

    Let **f** be a signal of length **L**, let ``c=dstii(f)`` and define the
    vector **w** of length **L** by

    .. w = [1 1 1 1 ... 1/sqrt(2)]

    .. math::

        w\\left(n\\right)=\\begin{cases}\\frac{1}{\\sqrt{2}} &
        \\text{if }n=L-1 \\\ 1 & \\text{otherwise}\\end{cases}

    Then

    .. math::

        c\\left(n+1\\right)=\\sqrt{\\frac{2}{L}}\\sum_{m=0}^{L-1}w\\left(
        n\\right)f\\left(m+1\\right)\\sin\\left(\\frac{\\pi}{L}n\\left(
        m+\\frac{1}{2}\\right)\\right)

    - Examples:

    The following figures show the first 4 basis functions of the DSTII of
    length 20:

    >>> import numpy as np
    >>> # The dstii is its own adjoint.
    >>> F = dstii(np.eye(20, dtype=np.float64))
    >>> import matplotlib.pyplot as plt
    >>> plt.close('all')
    >>> fig = plt.figure()
    >>> for ii in range(1,5):
    ...    ax = fig.add_subplot(4,1,ii)
    ...    ax.stem(F[:,ii-1])
    ...
    <Container object of 3 artists>
    <Container object of 3 artists>
    <Container object of 3 artists>
    <Container object of 3 artists>
    >>> plt.show()

    .. image:: images/dstii.png
       :width: 700px
       :alt: dstii image
       :align: center
    .. seealso::  :func:`~ltfatpy.fourier.dstiii`,
        :func:`~ltfatpy.fourier.dstiv`, :func:`~ltfatpy.fourier.dctii`

    - References:
        :cite:`rayi90,wi94`
    """
    (f, L, _, _, dim, permutedsize, order) = assert_sigreshape_pre(f, L, dim)
    if L is not None:
        f = postpad(f, L)
    if L == 1:
        c = f
    else:
        c = comp_dst(f, 2)
    return assert_sigreshape_post(c, dim, permutedsize, order)

if __name__ == '__main__':
    import doctest
    doctest.testmod()
