# -*- coding: utf-8 -*-
""" Module of frequency index of FFT modulations computation

Ported from ltfat_2.1.0/fourier/fftindex.m

.. moduleauthor:: Florent Jaillet
"""

from __future__ import print_function, division

import numpy as np


def fftindex(N, nyquistzero=False):
    """Frequency index of FFT modulations

    - Usage:

        | ``n = fftindex(N)``

    - Input parameters:

    :param int N: FFT length
    :param bool nyquistzero: If ``True``, sets the Nyquist frequency to zero

    - Output parameters:

    :returns: Indexes of the frequencies of the standard FFT
    :rtype: numpy.ndarray

    ``fftindex(N)`` returns the index of the frequencies of the standard FFT
    of length **N** as they are ordered in the output from the
    :func:`numpy.fft.fft` routine. The numbers returned are in the range
    ``-ceil(N/2)+1:floor(N/2)``.

    ``fftindex(N, True)`` does as above, but sets the Nyquist frequency to
    zero.

    .. seealso:: :func:`dft`
    """

    if not nyquistzero:
        if N % 2 == 0:
            n = np.r_[0:N//2+1, -N//2+1:0]
        else:
            n = np.r_[0:(N+1)//2, -(N-1)//2:0]
    else:
        if N % 2 == 0:
            n = np.r_[0:N//2, 0, -N//2+1:0]
        else:
            n = np.r_[0:(N+1)//2, -(N-1)//2:0]

    return n
