# -*- coding: utf-8 -*-
""" Module of signal resampling using Fourier interpolation

Ported from ltfat_2.1.0/fourier/fftresample.m

.. moduleauthor:: Florent Jaillet
"""

from __future__ import print_function, division

import numpy as np

from ltfatpy.comp.assert_sigreshape_pre import assert_sigreshape_pre
from ltfatpy.comp.assert_sigreshape_post import assert_sigreshape_post
from ltfatpy.tools.postpad import postpad
from ltfatpy.fourier.middlepad import middlepad
from ltfatpy.fourier.fftreal import fftreal
from ltfatpy.fourier.ifftreal import ifftreal


def fftresample(f, L, dim=None):
    """Resample signal using Fourier interpolation

    - Usage:
        | ``h = fftresample(f, L)``
        | ``h = fftresample(f, L, dim)``

    - Input parameters:

    :param numpy.ndarray f: Input array
    :param int L: Length of the output resampled array
    :param int dim: Axis over which to do the resampling

    - Output parameters:

    :returns: Resampled array
    :rtype: numpy.ndarray

    ``fftresample(f, L)`` returns a Fourier interpolation of the signal **f**
    to length **L**. If the function is applied to a matrix, it will apply
    to each column.

    ``fftresample(f, L, dim)`` does the same along dimension **dim**.

    If the input signal is *not* a periodic signal (or close to), the
    :func:`dctresample` method gives much better results at the endpoints.

    .. seealso:: :func:`dctresample`,
        :func:`~ltfatpy.fourier.middlepad.middlepad`
    """

    f, L, Ls, W, dim, permutedsize, order = assert_sigreshape_pre(f, L, dim)

    # The 'axis=0' and 'dim=0' below have been added to avoid fft and middlepad
    # being smart about choosing the dimension.
    if np.isrealobj(f):
        L2 = int(np.floor(L/2))+1
        # Note: There is a bug in the call of postpad below in ltfat 2.1.0. for
        # Octave. This bug has been reported and confirmed here:
        # http://sourceforge.net/p/ltfat/bugs/117/
        # This Python port corrects the bug by correclty padding with zeros
        # instead of ones in the Octave version.
        h = ifftreal(postpad(fftreal(f, dim=0), L2, dim=0), L, dim=0)/Ls*L
    else:
        h = np.fft.ifft(middlepad(np.fft.fft(f, axis=0), L, dim=0),
                        axis=0)/Ls*L

    h = assert_sigreshape_post(h, dim, permutedsize, order)

    return h
