# -*- coding: utf-8 -*-
""" Module of even function tests

Ported from ltfat_2.1.0/fourier/isevenfunction.m

.. moduleauthor:: Denis Arrivault
"""

from __future__ import print_function, division

import numpy as np
import numpy.linalg as LA


def isevenfunction(f, tol=1e-10, centering='wp'):
    """ True if function is even

    - Usage:

        | ``t = isevenfunction(f)``
        | ``t = isevenfunction(f, tol)``

    - Input parameters:

    :param numpy.ndarray f: vector of data to test (one dimension)
    :param float tol: tolerance (1e-10 by default)
    :param str centering: Point even function type : whole or half point even.
        **centering** can be 'wp' or 'hp', 'wp' is the default.

    -Output parameter:

    :return: True if **f** is whole point even
    :rtype: bool

    `isevenfunction(f)` returns True if *f* is whole point even. Otherwise it
    returns False.

    ``isevenfunction(f, tol)`` the same, using the tolerance *tol* to measure
    how large the error between the two parts of the vector can be. Default
    is 1e-10.

    Setting **centering** to 'hp', does the same for half point even functions.

    .. seealso:: :func:`~ltfatpy.fourier.middlepad.middlepad`, :func:`peven`
    """
    if f.ndim > 1:
        raise ValueError("f should be a one dimensional vector")

    # Define initial values for flags
    # definput.flags.centering = {'wp','hp'};
    # definput.keyvals.tol     = 1e-10;

    L = f.shape[0]

    if centering == 'wp':
        # Determine middle point of sequence.
        if L % 2 == 0:
            middle = L / 2
        else:
            middle = (L+1) / 2

        # Relative norm of difference between the parts of the signal.
        d = (LA.norm(f[1:middle] - np.conj(np.flipud(f[L-middle+1:L]))) /
             LA.norm(f))
    elif centering == 'hp':
        middle = np.floor(L/2)
        d = (LA.norm(f[0:middle] - np.conj(np.flipud(f[L-middle:L]))) /
             LA.norm(f))
    else:
        raise ValueError("centering parameter should be set to 'wp'" +
                         "(default) or 'hp'")

    # Return true if d less than tolerance.
    return (d <= tol)
