# -*- coding: utf-8 -*-
"""Module that computes Diagonal of Gabor frame operator

Ported from ltfat_2.1.0/gabor/gabframediag.m

.. moduleauthor:: Denis Arrivault
"""

from __future__ import print_function, division

import numpy as np

from ltfatpy.sigproc.fir2long import fir2long
from ltfatpy.gabor.dgtlength import dgtlength


def gabframediag(g, a, M, L):
    """Diagonal of Gabor frame operator

    - Usage:

        | ``d = gabframediag(g, a, M, L)``

    - Input parameters:

    :param numpy.ndarray g: Window function
    :param int a: Length of time shift
    :param int M: Number of channels
    :param int L: Length of transform to do

    - Output parameters:

    :returns: Diagonal stored as a column vector
    :rtype: numpy.ndarray

    ``gabframediag(g, a, M, L)`` computes the diagonal of the Gabor frame
    operator with respect to the window **g** and parameters **a** and **M**.
    The diagonal is stored a as column vector of length **L**.

    The diagonal of the frame operator can for instance be used as a
    preconditioner.

    .. seealso:: :func:`~ltfatpy.gabor.dgt.dgt`
    """
    # Verify a, M and get L
    Luser = dgtlength(L, a, M)
    if L != Luser:
        raise ValueError(("Incorrect transform length L={0:d} specified." +
                          " Next valid length  is L={0:d}. See the help" +
                          " of DGTLENGTH for the requirements.").format(L,
                         Luser))

    # Determine the window
    (g, info) = _call_gabwin(g, a, M, L)
    if L < info['gl']:
        raise ValueError('Window is too long.')

    # compute the diagonal
    glong2 = np.abs(fir2long(g, L))**2
    N = L/a

    # The diagonal is a-periodic, so compute a single period by summing up
    # glong2 in slices.
    d = np.tile(np.sum(glong2.reshape(N, a), axis=0), (N,)) * M
    return d


def _call_gabwin(g, a, M, L):
    """ Internal use only """

    from ltfatpy.gabor.gabwin import gabwin
    return gabwin(g, a, M, L)
