# -*- coding: utf-8 -*-
"""Module of phaseunlocking

Ported from ltfat_2.1.0/gabor/phaseunlock.m

.. moduleauthor:: Florent Jaillet
"""

from __future__ import print_function, division

import six
import numpy as np


def phaseunlock(c, a):
    """Undo phase lock of Gabor coefficients

    - Usage:

        | ``c_out = phaseunlock(c, a)``

    - Input parameters:

    :param numpy.ndarray c: phaselocked Gabor coefficients
    :param int a: Length of time shift

    - Output parameters:

    :returns: non-phaselocked Gabor coefficients
    :rtype: numpy.ndarray

    ``phaseunlock(c, a)`` removes phase locking from the Gabor coefficients
    **c**.
    The coefficient must have been obtained from a
    :func:`~ltfatpy.gabor.dgt.dgt` with parameter **a**.

    Phase locking the coefficients modifies them so as if they were obtained
    from a time-invariant Gabor system. A filter bank produces phase locked
    coefficients.

    .. seealso:: :func:`~ltfatpy.gabor.dgt.dgt`,
                 :func:`~ltfatpy.gabor.phaselock.phaselock`, :func:`symphase`

    - References:
        :cite:`puc95`
    """

    # NOTE: This function doesn't support the parameter lt (lattice type)
    # supported by the corresponding octave function and the lattice used is
    # seperable (square lattice lt = (0, 1)).

    if not isinstance(a, six.integer_types):
        raise(TypeError('a must be an integer'))

    M = c.shape[0]
    N = c.shape[1]
    L = N * a
    b = L / M

    if b % 1 != 0.:
        raise(ValueError('Lattice error. The a parameter is probably '
                         'incorrect.'))

    TimeInd = np.arange(N) * a
    FreqInd = np.arange(M)

    phase = FreqInd[:, np.newaxis].dot(TimeInd[np.newaxis, :])
    phase = np.mod(phase, M)
    phase = np.exp(-2.*1.j*np.pi*phase/M)

    # Handle multisignals
    shape = np.array(c.shape)
    if shape.shape[0] > 2:
        shape[2:] = 1
    c_out = c * phase.reshape(shape)

    return c_out
