# -*- coding: utf-8 -*-
"""Module of S0 norm calculation

Ported from ltfat_2.1.0/gabor/s0norm.m

.. moduleauthor:: Denis Arrivault
"""

from __future__ import print_function, division

import numpy as np
from numpy import linalg as LA

from ltfatpy.comp.assert_sigreshape_pre import assert_sigreshape_pre
from ltfatpy.comp.assert_sigreshape_post import assert_sigreshape_post


def s0norm(f, rel=False, dim=None):
    """S0-norm of signal

    - Usage:

        | ``y = s0norm(f)``

    - Input parameters:

    :param numpy.ndarray f: is the signal
    :param bool rel: True if returning result should be relative to the
                        :math:`l^2` norm (the energy) of the signal.
                        False by default
    :param int dim: dimension along which norm is applied (first
                    non-singleton dimension as default)

    - Output parameters:

    :return: s0-norm
    :rtype: float

    ``s0norm(f)`` computes the :math:`S_0`-norm of a vector.

    If the input is a matrix or ND-array, the :math:`S_0`-norm is computed
    along the first (non-singleton) dimension, and a vector of values is
    returned.

    .. warning:: The :math:`S_0`-norm is computed by computing a full
        short-time Fourier transform of a signal, which can be quite
        time-consuming. Use this function with care for long signals.
    """
    #  ------ Computation --------------------------
    (f, L, _unused, W, dim, permutedsize, order) = assert_sigreshape_pre(
                                                    f, dim=dim)
    permutedsize_list = [1] + list(permutedsize[1:])
    permutedsize = tuple(permutedsize_list)
    if len(permutedsize_list) == 1:
        permutedsize_list.append(1)
    y = np.zeros(tuple(permutedsize_list), dtype=f.dtype)
    g = __call_pgauss(L)

    for ii in range(W):
        # Compute the STFT by the simple algorithm and sum each column of the
        # STFT as they are computed, to avoid L^2 memory usage.
        for jj in range(L):
            y[0, ii] = y[0, ii] + np.sum(np.abs(np.fft.fft(f[:, ii] *
                                         np.roll(g, jj, axis=0))))

        if rel:
            y[0, ii] = y[0, ii] / LA.norm(f[:, ii])

    y /= L
    y = assert_sigreshape_post(y, dim, permutedsize, order)
    return y


def __call_pgauss(L):
    from ltfatpy.fourier.pgauss import pgauss
    return pgauss(L)[0]
