# -*- coding: utf-8 -*-
""" Module of N largest coefficients extraction

Ported from ltfat_2.1.0/sigproc/largestn.m

.. moduleauthor:: Florent Jaillet
"""

from __future__ import print_function, division

import six
import numpy as np

from ltfatpy.sigproc.thresh import thresh


def largestn(xi, N, thresh_type='hard'):
    """Keep N largest coefficients

    - Usage:

        | ``(xo, Nout) = largestn(xi, N)``
        | ``(xo, Nout) = largestn(xi, N, thresh_type)``

    - Input parameters:

    :param numpy.ndarray xi: Input array
    :param int N: Number of kept coefficients
    :param str thresh_type: Optional flag specifying the type of thresholding
        (see possible values below)

    - Output parameters:

    :returns: ``(xo, Nout)``
    :rtype: tuple

    :var numpy.ndarray xo: Array of the same shape as **xi** keeping
        the **N** largest coefficients
    :var int Nout: Number of coefficients kept

    The parameter **thresh_type** can take the following values:
        ============ ======================================================
        ``'hard'``   Perform hard thresholding. This is the default.

        ``'wiener'`` Perform empirical Wiener shrinkage. This is in between
                     soft and hard thresholding.

        ``'soft'``   Perform soft thresholding.
        ============ ======================================================

    If the coefficients represents a signal expanded in an orthonormal
    basis then this will be the best N-term approximation.

    .. note::
        If soft- or Wiener thresholding is selected, only ``N-1``
        coefficients will actually be returned. This is caused by the Nth
        coefficient being set to zero.

    .. seealso::
        :func:`~ltfatpy.sigproc.largestr.largestr`

    - References:
        :cite:`ma98`
    """

    if not isinstance(N, six.integer_types):
        raise TypeError('N must be an int.')

    # Sort the absolute values of the coefficients.
    sxi = np.sort(abs(xi.flatten()))

    # Find the coefficient sitting at position N through the array,
    # and use this as a threshing value.
    if N <= 0:
        # Choose a thresh value higher than max
        lamb = sxi[-1] + 1.
    else:
        lamb = sxi[-N]

    xo, Nout = thresh(xi, lamb, thresh_type)

    return (xo, Nout)
