# -*- coding: utf-8 -*-
"""Read .mat data files generated with the Matlab version of Ltfat and
containing signals, dgt and idgt for validations.

.. moduleauthor:: Denis Arrivault
"""

from __future__ import print_function, division

import scipy.io


class DgtSignals:
    """This class opens the file and provides :func:`read_next_signal`
    method for reading each (signal, window, dgt, dual window,
    idgt) tuples contained in the file.
    """

    def __init__(self, filename):
        """Constructor

        - Input parameter:

        :param str filename: Name of the matlab data file
        """
        self.Data = scipy.io.loadmat(filename, struct_as_record=False,
                                     squeeze_me=True)['Data']
        self.ind = 0
        self.size = self.Data.shape[0]

    def read_next_signal(self):
        """Read the next signal of the file.

        - Usage:

            | :literal:`(TYPE, PHASETYPE, L, W, a, M, gl, SIGNAL, WINDOW, DGT,
                DUAL_WINDOW, IDGT) = DgtSignals("file.mat").read_next_signal()`

        - Output parameters:

        :returns:
            :literal:`(TYPE, PHASETYPE, L, W, a, M, gl, SIGNAL, WINDOW,
            DGT, DUAL_WINDOW, IDGT)`

            Empty strings if all signals have been read.
        """
        if(self.ind >= self.size):
            return ('', '', '', '', '', '', '', '', '', '', '', '')

        TYPE = str(self.Data[self.ind].rname)
        PHASETYPE = str(self.Data[self.ind].phase)
        L = self.Data[self.ind].L
        W = self.Data[self.ind].W
        a = self.Data[self.ind].a
        M = self.Data[self.ind].M
        gl = self.Data[self.ind].gl
        SIGNAL = self.Data[self.ind].f
        WINDOW = self.Data[self.ind].g
        DGT = self.Data[self.ind].cc
        DUAL_WINDOW = self.Data[self.ind].gd
        IDGT = self.Data[self.ind].freq
        self.ind += 1
        return (TYPE, PHASETYPE, L, W, a, M, gl, SIGNAL, WINDOW, DGT,
                DUAL_WINDOW, IDGT)
