# -*- coding: utf-8 -*-
"""Test of the ifftreal function

.. moduleauthor:: Florent Jaillet
"""

from __future__ import print_function, division

import unittest
import numpy as np
from numpy.testing import assert_allclose

from ltfatpy.fourier.ifftreal import ifftreal
from ltfatpy.fourier.fftreal import fftreal
from ltfatpy.tests.datasets.read_ref_mat import read_ref_mat
from ltfatpy.tests.datasets.get_dataset_path import get_dataset_path


class TestIfftreal(unittest.TestCase):

    # Called before the tests.
    def setUp(self):
        print('\nStart TestIfftreal')

    # Called after the tests.
    def tearDown(self):
        print('Test done')

    def test_shape(self):
        """Check that the output has the expected shape
        """
        shapes = ((4,), (4, 3), (4, 3, 2))
        N_val = (range(2, 10))
        inputs = {}
        for shape in shapes:
            inputs['c'] = (np.random.random(shape) +
                           1.0j * np.random.random(shape))
            for dim in range(len(shape)):
                inputs['dim'] = dim
                for N in N_val:
                    inputs['N'] = N
                    f = ifftreal(**inputs)
                    expected_shape = list(shape)
                    expected_shape[dim] = N
                    expected_shape = tuple(expected_shape)
                    msg = ('Wrong shape in the ouput of ifftreal with ' +
                           'inputs ' + str(inputs))
                    self.assertEqual(f.shape, expected_shape, msg)

    def test_inversion(self):
        """Check that ifftreal is the inverse of fftreal
        """
        K = 16
        shapes = ((K,), (K, K-1), (K, K-1, K-2))
        inputs = {}
        for shape in shapes:
            f_ref = np.random.random(shape)
            for dim in range(len(shape)):
                inputs['dim'] = dim
                N_val = (range(shape[dim], 2*shape[dim]))
                for N in N_val:
                    inputs['N'] = N
                    c = fftreal(f_ref, **inputs)
                    f = ifftreal(c, **inputs)
                    f = np.take(f, np.arange(f_ref.shape[dim]), dim)
                    msg = ('Result of inversion is not close enough to the ' +
                           'original when composing fftreal and ifftreal ' +
                           'with inputs ' + str(inputs))
                    assert_allclose(f, f_ref, rtol=1e-10, atol=1e-14,
                                    err_msg=msg)

    def test_known(self):
        """Checking ifftreal on some known results taken from Octave
        """
        filename = get_dataset_path('ifftreal_ref.mat')
        data = read_ref_mat(filename)

        for inputs, outputs in data:
            h = ifftreal(**inputs)
            msg = ('Wrong value in output of ifftreal with inputs ' +
                   str(inputs))
            assert_allclose(h, outputs[0], rtol=5e-13, atol=1e-15, err_msg=msg)


if __name__ == '__main__':
    suite = unittest.TestLoader().loadTestsFromTestCase(TestIfftreal)
    unittest.TextTestRunner(verbosity=2).run(suite)
