# -*- coding: utf-8 -*-
"""Test of the phaselock function

.. moduleauthor:: Florent Jaillet
"""

from __future__ import print_function, division

import unittest
import numpy as np
from numpy.testing import assert_array_equal, assert_allclose

from ltfatpy.gabor.phaselock import phaselock
from ltfatpy.gabor.phaseunlock import phaseunlock
from ltfatpy.gabor.dgt import dgt
from ltfatpy.tests.datasets.read_ref_mat import read_ref_mat
from ltfatpy.tests.datasets.get_dataset_path import get_dataset_path

# NOTE: The reference values used in the tests correspond to results
# obtained with Octave using ltfat 2.1.0


class TestPhaselock(unittest.TestCase):

    # Called before the tests.
    def setUp(self):
        print('\nStart TestPhaselock')

    # Called after the tests.
    def tearDown(self):
        print('Test done')

    def test_exceptions(self):
        """Check that the right exceptions are raised when expected
        """
        c = dgt(np.random.random((16,)), 'gauss', 2, 6)[0]
        # a is expected to be an integer, check that we get type error if not
        self.assertRaises(TypeError, phaselock, c, 'test')
        self.assertRaises(TypeError, phaselock, c, 1.0)
        self.assertRaises(TypeError, phaselock, c, (1, 2))
        # a should be the value used to generate c with dgt, check that we get
        # a value error if not
        self.assertRaises(ValueError, phaselock, c, 3)

    def test_known(self):
        """Check phaselock on some known results taken from Octave
        """
        filename = get_dataset_path('phaselock_ref.mat')
        data = read_ref_mat(filename)

        for inputs, outputs in data:
            out = phaselock(**inputs)
            msg = ('Wrong output in phaselock with inputs ' +
                   str(inputs))
            assert_array_equal(out, outputs[0], msg)

    def test_inverse(self):
        """Check that phaselock and phaseunlock are inverse functions
        """
        shapes = ((3, 3), (12, 6), (13, 13))
        a_val = (1, 4, 7)
        inputs = {}
        for shape, a in zip(shapes, a_val):
            inputs['c'] = np.random.random(shape)
            inputs['a'] = a
            msg = ('phaselock and phaseunlock should be inverse with inputs ' +
                   str(inputs))
            out = phaseunlock(phaselock(**inputs), inputs['a'])
            assert_allclose(inputs['c'], out, rtol=1e-15, err_msg=msg)
            out = phaselock(phaseunlock(**inputs), inputs['a'])
            assert_allclose(inputs['c'], out, rtol=1e-15, err_msg=msg)


if __name__ == '__main__':
    suite = unittest.TestLoader().loadTestsFromTestCase(TestPhaselock)
    unittest.TextTestRunner(verbosity=2).run(suite)
