# -*- coding: utf-8 -*-
"""Test of the sgram function

NOTE: The validity of the plotting features of sgram are not tested here, only
the fact that they can run without error is tested.

.. moduleauthor:: Florent Jaillet
"""

from __future__ import print_function, division

import unittest
import numpy as np
from numpy.testing import assert_array_equal, assert_allclose

from ltfatpy.gabor.sgram import sgram
from ltfatpy.tests.datasets.read_ref_mat import read_ref_mat
from ltfatpy.tests.datasets.get_dataset_path import get_dataset_path

# NOTE: The reference values used in the tests correspond to results
# obtained with Octave using ltfat 2.1.0


class TestSgram(unittest.TestCase):

    # Called before the tests.
    def setUp(self):
        print('\nStart TestSgram')

    # Called after the tests.
    def tearDown(self):
        print('Test done')

    def test_exceptions(self):
        """Check that the right exceptions are raised when expected
        """
        # coef must be a 1D numpy.ndarray, check that we get the right
        # exceptions if not
        self.assertRaises(TypeError, sgram, 'test')
        self.assertRaises(TypeError, sgram, 1)
        self.assertRaises(TypeError, sgram, 1.)
        self.assertRaises(TypeError, sgram, [0, 2])
        self.assertRaises(ValueError, sgram, np.ones((4, 2)))

        # wrong values of xres and yres should lead to exceptions
        self.assertRaises(ValueError, sgram, np.random.random((20, )), xres=3,
                          yres=3)
        self.assertRaises(TypeError, sgram, np.random.random((20, )), xres='a',
                          yres=3)

    def test_working(self):
        """Check that sgram is working as expected
        """
        filename = get_dataset_path('sgram_ref.mat')
        data = read_ref_mat(filename)

        inputs, outputs = data[0]
        out = sgram(**inputs)
        msg = 'Wrong value in output of sgram with inputs ' + str(inputs)
        assert_allclose(out, outputs[0], rtol=2e-15, err_msg=msg)

        for inputs, outputs in data[1:]:
            out = sgram(**inputs)
            msg = ('Wrong value in output of sgram with inputs ' +
                   str(inputs))
            assert_array_equal(out, outputs[0], msg)


if __name__ == '__main__':
    suite = unittest.TestLoader().loadTestsFromTestCase(TestSgram)
    unittest.TextTestRunner(verbosity=2).run(suite)
