# -*- coding: utf-8 -*-
"""Test of the postpad function

.. moduleauthor:: Florent Jaillet
"""

from __future__ import print_function, division

import unittest
import numpy as np

from ltfatpy.tools.postpad import postpad


class TestPostpad(unittest.TestCase):

    # Called before the tests.
    def setUp(self):
        print('\nStart TestPostpad')

    # Called after the tests.
    def tearDown(self):
        print('Test done')

    def test_working(self):
        """Check that postpad works as expected
        """
        inputs = {}
        N = 5
        inputs['x'] = np.arange(N, dtype=float)

        # testing padding
        inputs['L'] = 10
        y = postpad(**inputs)
        msg = ('Wrong shape in postpad output in the padding case using '
               'inputs ' + str(inputs))
        self.assertEqual(y.shape, (inputs['L'],), msg)
        for val_x, val_y in zip(inputs['x'], y[:N]):
            msg = ('Wrong output value in postpad, y should be equal to x in '
                   'the non-padded part. Inputs are ' + str(inputs))
            self.assertEqual(val_x, val_y, msg)
        for val in y[N:]:
            msg = ('Wrong output value in postpad, y should be equal to 0. in '
                   'the padded part. Inputs are ' + str(inputs))
            self.assertEqual(val, 0., msg)

        # testing truncation
        inputs['L'] = 3
        y = postpad(**inputs)
        msg = ('Wrong shape in postpad output in the truncation case using '
               'inputs ' + str(inputs))
        self.assertEqual(y.shape, (inputs['L'],), msg)
        for val_x, val_y in zip(inputs['x'][:inputs['L']], y):
            msg = ('Wrong output value in postpad, y should be equal to x in '
                   'the non-truncated part. Inputs are ' + str(inputs))
            self.assertEqual(val_x, val_y, msg)

    def test_default_val(self):
        """Check that expected default values are used
        """
        # check that default value for C is 0.
        inputs = {}
        N = 3
        inputs['x'] = np.ones(N)
        inputs['L'] = N+1
        y = postpad(**inputs)
        msg = 'Wrong default value for C using inputs ' + str(inputs)
        self.assertEqual(y[-1], 0., )

        # check that default dim is the first non-singleton dimension
        N = 3
        inputs['L'] = N+1
        x_shape = (1, 1, N, 5)
        inputs['x'] = np.ones(x_shape)
        y = postpad(**inputs)
        y_exp_shape = (1, 1, inputs['L'], 5)
        msg = 'Wrong default value for dim using inputs ' + str(inputs)
        self.assertEqual(y.shape, y_exp_shape, msg)

    def test_param_C(self):
        """Check that the value of C is taken into account
        """
        inputs = {}
        N = 3
        inputs['C'] = 2.
        inputs['x'] = np.ones(N)
        inputs['L'] = N+1
        y = postpad(**inputs)
        msg = 'Postpad is misusing C with inputs ' + str(inputs)
        self.assertEqual(y[-1], inputs['C'], msg)

    def test_param_dim(self):
        """Check that dim is taken into account
        """
        inputs = {}
        N = 3
        M = 4
        inputs['L'] = 5
        inputs['x'] = np.ones((N, M))
        inputs['dim'] = 0
        y = postpad(**inputs)
        msg = 'postpad is misusing dim with inputs ' + str(inputs)
        self.assertEqual(y.shape, (inputs['L'], M), msg)
        inputs['dim'] = 1
        y = postpad(**inputs)
        msg = 'postpad is misusing dim with inputs ' + str(inputs)
        self.assertEqual(y.shape, (N, inputs['L']), msg)


if __name__ == '__main__':
    suite = unittest.TestLoader().loadTestsFromTestCase(TestPostpad)
    unittest.TextTestRunner(verbosity=2).run(suite)
