# -*- coding: utf-8 -*-
"""This module pads or truncates arrays

Ported from ltfat_2.1.0/mex/postpad.m

.. moduleauthor:: Florent Jaillet
"""

from __future__ import print_function, division

import numpy as np

from ltfatpy.comp.assert_sigreshape_pre import assert_sigreshape_pre
from ltfatpy.comp.assert_sigreshape_post import assert_sigreshape_post


def postpad(x, L, C=0., dim=None):
    """Pads or truncates an array to a specified length

    - Usage:

        | ``y = postpad(x, L)``
        | ``y = postpad(x, L, C)``
        | ``y = postpad(x, L, C, dim)``

    - Input parameters:

    :param numpy.ndarray x: Input array
    :param int L: Length of the output array
    :param float C: Value of the padded entries
    :param int dim: Axis over which to pad or truncate **x**

    - Ouput parameters:

    :returns: Padded or truncated array
    :rtype: numpy.ndarray

    ``postpad(x, L)`` will add zeros to the end of the vector **x**, until
    the result has length **L**. If **L** is less than the length of the
    signal, it will be truncated. ``postpad`` works along the first
    non-singleton dimension.

    ``postpad(x, L, C)`` will add entries with a value of **C** instead of
    zeros.

    ``postpad(x, L, C, dim)`` works along dimension **dim** instead of the
    first non-singleton.

    .. seealso::
        :func:`~ltfatpy.fourier.middlepad.middlepad`
    """

    if dim is None:
        # by default, dim is the first non-singleton dimension
        dim = np.nonzero(np.array(x.shape) > 1)[0][0]

    x, L, Ls, W, dim, permutedsize, order = assert_sigreshape_pre(x, L, dim)

    if Ls < L:
        tmp = np.empty((L-Ls, W), dtype=x.dtype)
        tmp[:] = C
        y = np.concatenate((x, tmp), axis=0)
    else:
        y = x[:L, :].copy()

    y = assert_sigreshape_post(y, dim, permutedsize, order)

    return y
